@@ -9,9 +9,13 @@
 
 """
 
-Internally, CliMetLab cache is managed by the module `climetlab.core.cache`, it relies on a sqlite database. The :py:func:`cache_file` function provide a unique path for a given couple (`owner`, `args`). The calling code is responsible for checking if the file exists and decide to read it or create it.
+Internally, CliMetLab cache is managed by the module `climetlab.core.cache`,
+it relies on a sqlite database. The :py:func:`cache_file` function provide
+a unique path for a given couple (`owner`, `args`).
+The calling code is responsible for checking if the file exists and
+decide to read it or create it.
 
-"""  # noqa: E501
+"""
 
 import datetime
 import hashlib
@@ -23,11 +27,13 @@ import sqlite3
 import threading
 from functools import wraps
 
+import psutil
+
 from climetlab.core.settings import SETTINGS
 from climetlab.utils import bytes_to_string
 from climetlab.utils.html import css
 
-VERSION = 1
+VERSION = 2
 CACHE_DB = f"cache-{VERSION}.db"
 
 LOG = logging.getLogger(__name__)
@@ -91,7 +97,6 @@ class Cache(threading.Thread):
         self._connection = None
         self._queue = []
         self._condition = threading.Condition()
-        SETTINGS.on_change(in_executor(self._settings_changed))
 
     def run(self):
         while True:
@@ -109,6 +114,7 @@ class Cache(threading.Thread):
             if not os.path.exists(cache_dir):
                 os.makedirs(cache_dir, exist_ok=True)
             cache_db = os.path.join(cache_dir, CACHE_DB)
+            LOG.debug("Cache database is %s", cache_db)
             self._connection = sqlite3.connect(cache_db)
             # So we can use rows as dictionaries
             self._connection.row_factory = sqlite3.Row
@@ -122,11 +128,11 @@ class Cache(threading.Thread):
                         args          TEXT NOT NULL,
                         creation_date TEXT NOT NULL,
                         flags         INTEGER DEFAULT 0,
-                        remote_date   TEXT, -- TODO expire URLs
-                        remote_tag    TEXT, -- TODO expire URLs
-                        last_access   TEXT,
+                        owner_data    TEXT,
+                        last_access   TEXT NOT NULL,
                         type          TEXT,
                         parent        TEXT,
+                        replaced      TEXT,
                         extra         TEXT,
                         expires       INTEGER,
                         accesses      INTEGER,
@@ -141,7 +147,15 @@ class Cache(threading.Thread):
             self._condition.notify_all()
             return s
 
+    def _file_in_cache_directory(self, path):
+        cache_directory = SETTINGS.get("cache-directory")
+        return path.startswith(cache_directory)
+
+    def _ensure_in_cache(self, path):
+        assert self._file_in_cache_directory(path), f"File not in cache {path}"
+
     def _settings_changed(self):
+        LOG.debug("Settings changed")
         self._connection = None  # The user may have changed the cache directory
         self._check_cache_size()
 
@@ -164,16 +178,23 @@ class Cache(threading.Thread):
         with self.connection as db:
             db.execute("DELETE FROM cache WHERE owner=?", (owner,))
 
-    def _get_cached_files(self):
+    def _cache_entries(self):
         result = []
         with self.connection as db:
             for n in db.execute("SELECT * FROM cache").fetchall():
                 n = dict(n)
                 n["args"] = json.loads(n["args"])
-                result.append(n)
+                try:
+                    n["owner_data"] = json.loads(n["owner_data"])
+                except Exception:
+                    pass
+                if os.path.exists(n["path"]):
+                    result.append(n)
         return result
 
-    def _update_entry(self, path):
+    def _update_entry(self, path, owner_data=None):
+        self._ensure_in_cache(path)
+
         if os.path.isdir(path):
             kind = "directory"
             size = 0
@@ -186,8 +207,13 @@ class Cache(threading.Thread):
 
         with self.connection as db:
             db.execute(
-                "UPDATE cache SET size=?, type=? WHERE path=?",
-                (size, kind, path),
+                "UPDATE cache SET size=?, type=?, owner_data=? WHERE path=?",
+                (
+                    size,
+                    kind,
+                    json.dumps(owner_data),
+                    path,
+                ),
             )
 
     def _update_cache(self, clean=False):
@@ -219,7 +245,7 @@ class Cache(threading.Thread):
             if update or commit:
                 db.commit()
 
-    def _find_orphans(self):
+    def _housekeeping(self):
         top = SETTINGS.get("cache-directory")
         with self.connection as db:
             for name in os.listdir(top):
@@ -231,50 +257,82 @@ class Cache(threading.Thread):
                     "SELECT count(*) FROM cache WHERE path=?", (full,)
                 ).fetchone()[0]
 
-                if count == 0:
+                if count > 0:
+                    continue
 
-                    parent = None
-                    start = full.split(".")[0] + "%"
-                    for n in db.execute(
-                        "SELECT path FROM cache WHERE parent IS NULL and path LIKE ?",
-                        (start,),
-                    ).fetchall():
-                        if full.startswith(n["path"]):
-                            parent = n["path"]
-                            break
-                    if not parent:
-                        LOG.warning(f"CliMetLab cache: orphan found: {full}")
+                parent = None
+                start = full.split(".")[0] + "%"
+                for n in db.execute(
+                    "SELECT path FROM cache WHERE parent IS NULL and path LIKE ?",
+                    (start,),
+                ).fetchall():
+                    if full.startswith(n["path"]):
+                        parent = n["path"]
+                        break
 
-                    self._register_cache_file(
-                        full,
-                        "orphans",
-                        None,
-                        parent,
+                if parent is None:
+                    LOG.warning(f"CliMetLab cache: orphan found: {full}")
+                else:
+                    LOG.debug(
+                        f"CliMetLab cache: orphan found: {full} with parent {parent}"
                     )
 
-    def _delete_entry(self, db, top, entry):
-        path, size, owner, args = (
-            entry["path"],
-            entry["size"],
-            entry["owner"],
-            entry["args"],
-        )
-        assert path.startswith(top), (path, top)
+                self._register_cache_file(
+                    full,
+                    "orphans",
+                    None,
+                    parent,
+                )
+        self._update_cache()
+
+    def _delete_file(self, path):
+
+        self._ensure_in_cache(path)
+
+        try:
+            if os.path.isdir(path) and not os.path.islink(path):
+                shutil.rmtree(path)
+            else:
+                os.unlink(path)
+        except Exception:
+            LOG.exception("Deleting %s", path)
+
+    def _delete_entry(self, entry):
+        if isinstance(entry, str):
+            path, size, owner, args = entry, None, None, None
+            try:
+                size = os.path.getsize(path)
+            except OSError:
+                pass
+        else:
+            path, size, owner, args = (
+                entry["path"],
+                entry["size"],
+                entry["owner"],
+                entry["args"],
+            )
+
+        total = 0
+
+        # First, delete child files, e.g. unzipped data
+        with self.connection as db:
+            for child in db.execute("SELECT * FROM cache WHERE parent = ?", (path,)):
+                total += self._delete_entry(child)
+
         if not os.path.exists(path):
             LOG.warning(f"cache file lost: {path}")
             with self.connection as db:
                 db.execute("DELETE FROM cache WHERE path=?", (path,))
-            return 0
-        delete = path + ".delete"
-        os.rename(path, delete)
+            return total
+
         LOG.warning(f"CliMetLab cache: deleting {path} ({bytes_to_string(size)})")
         LOG.warning(f"CliMetLab cache: {owner} {args}")
-        if os.path.isdir(delete):
-            shutil.rmtree(delete)
-        else:
-            os.unlink(delete)
-        db.execute("DELETE FROM cache WHERE path=?", (path,))
-        return size
+        self._delete_file(path)
+
+        with self.connection as db:
+            db.execute("DELETE FROM cache WHERE path=?", (path,))
+
+        return total + size
 
     def _decache(self, bytes):
         # _find_orphans()
@@ -286,7 +344,7 @@ class Cache(threading.Thread):
         LOG.warning("CliMetLab cache: trying to free %s", bytes_to_string(bytes))
 
         total = 0
-        top = SETTINGS.get("cache-directory")
+
         with self.connection as db:
 
             latest = self._latest_date()
@@ -296,7 +354,7 @@ class Cache(threading.Thread):
                 "SELECT * FROM cache WHERE size IS NOT NULL AND creation_date < ? ORDER BY last_access ASC",
             ):
                 for entry in db.execute(stmt, (latest,)):
-                    total += self._delete_entry(db, top, entry)
+                    total += self._delete_entry(entry)
                     if total >= bytes:
                         LOG.warning(
                             "CliMetLab cache: freed %s from cache",
@@ -324,6 +382,8 @@ class Cache(threading.Thread):
             None or False if database does not need to be updated. TODO: clarify.
         """
 
+        self._ensure_in_cache(path)
+
         with self.connection as db:
 
             now = datetime.datetime.utcnow()
@@ -357,10 +417,9 @@ class Cache(threading.Thread):
                     (path, owner, args, now, now, 1, parent),
                 )
 
-            else:
-                assert parent is None
-
-        return not changes
+            return dict(
+                db.execute("SELECT * FROM cache WHERE path=?", (path,)).fetchone()
+            )
 
     def _cache_size(self):
         with self.connection as db:
@@ -369,12 +428,29 @@ class Cache(threading.Thread):
                 size = 0
             return size
 
+    def _decache_file(self, path):
+        self._delete_entry(path)
+
     def _check_cache_size(self):
-        maximum = SETTINGS.as_bytes("maximum-cache-size")
+
+        # Check absolute limit
         size = self._cache_size()
-        if size > maximum:
+        maximum = SETTINGS.get("maximum-cache-size")
+        if maximum is not None and size > maximum:
+            self._housekeeping()
             self._decache(size - maximum)
 
+        # Check relative limit
+        size = self._cache_size()
+        usage = SETTINGS.get("maximum-cache-disk-usage")
+        cache_directory = SETTINGS.get("cache-directory")
+        df = psutil.disk_usage(cache_directory)
+        if df.percent > usage:
+            LOG.debug("Cache disk usage %s, limit %s", df.percent, usage)
+            self._housekeeping()
+            delta = (df.percent - usage) * df.total * 0.01
+            self._decache(delta)
+
     def _repr_html_(self):
         """Return a html representation of the cache .
 
@@ -390,7 +466,7 @@ class Cache(threading.Thread):
                 html.append("<table class='climetlab'>")
                 html.append("<td><td colspan='2'>%s</td></tr>" % (n["path"],))
 
-                for k in [x for x in n.keys() if x != "path"]:
+                for k in [x for x in n.keys() if x not in ("path", "owner_data")]:
                     v = bytes_to_string(n[k]) if k == "size" else n[k]
                     html.append("<td><td>%s</td><td>%s</td></tr>" % (k, v))
                 html.append("</table>")
@@ -405,11 +481,23 @@ register_cache_file = in_executor(CACHE._register_cache_file)
 update_entry = in_executor(CACHE._update_entry)
 check_cache_size = in_executor_forget(CACHE._check_cache_size)
 cache_size = in_executor(CACHE._cache_size)
-get_cached_files = in_executor(CACHE._get_cached_files)
+cache_entries = in_executor(CACHE._cache_entries)
 purge_cache = in_executor(CACHE._purge_cache)
+housekeeping = in_executor(CACHE._housekeeping)
+decache_file = in_executor(CACHE._decache_file)
+file_in_cache_directory = in_executor(CACHE._file_in_cache_directory)
+settings_changed = in_executor(CACHE._settings_changed)
 
 
-def cache_file(owner: str, create, args, extension: str = ".cache"):
+def cache_file(
+    owner: str,
+    create,
+    args,
+    hash_extra=None,
+    extension: str = ".cache",
+    force=None,
+    replace=None,
+):
     """Creates a cache file in the climetlab cache-directory (defined in the :py:class:`Settings`).
     Uses :py:func:`_register_cache_file()`
 
@@ -426,39 +514,51 @@ def cache_file(owner: str, create, args, extension: str = ".cache"):
         Full path to the cache file.
     """
 
-    def _update_hash(m, x):
-
-        if isinstance(x, (list, tuple)):
-            for y in x:
-                _update_hash(m, y)
-            return
-
-        if isinstance(x, dict):
-            for k, v in sorted(x.items()):
-                _update_hash(m, k)
-                _update_hash(m, v)
-            return
-
-        m.update(str(x).encode("utf-8"))
-
     m = hashlib.sha256()
-    _update_hash(m, owner)
-    _update_hash(m, args)
+    m.update(owner.encode("utf-8"))
+    m.update(json.dumps(args, sort_keys=True).encode("utf-8"))
+    m.update(json.dumps(hash_extra, sort_keys=True).encode("utf-8"))
+    m.update(json.dumps(extension, sort_keys=True).encode("utf-8"))
 
-    path = "%s/%s-%s%s" % (
+    if replace is not None:
+        # Don't replace files that are not in the cache
+        if not file_in_cache_directory(replace):
+            replace = None
+
+    path = os.path.join(
         SETTINGS.get("cache-directory"),
-        owner.lower(),
-        m.hexdigest(),
-        extension,
+        "{}-{}{}".format(
+            owner.lower(),
+            m.hexdigest(),
+            extension,
+        ),
     )
 
-    register_cache_file(path, owner, args)
+    record = register_cache_file(path, owner, args)
+    if os.path.exists(path):
+        if callable(force):
+            owner_data = record["owner_data"]
+            if owner_data is not None:
+                owner_data = json.loads(owner_data)
+            force = force(args, path, owner_data)
+
+        if force:
+            decache_file(path)
 
     if not os.path.exists(path):
 
-        create(path + ".tmp", args)
-        os.rename(path + ".tmp", path)
-        update_entry(path)
+        tmp = ".{}-{}.tmp".format(os.getpid(), threading.get_ident())
+
+        owner_data = create(path + tmp, args)
+
+        os.rename(path + tmp, path)
+
+        update_entry(path, owner_data)
+
         check_cache_size()
 
     return path
+
+
+# housekeeping()
+SETTINGS.on_change(settings_changed)
