@@ -7,6 +7,7 @@
 # nor does it submit to any jurisdiction.
 #
 
+import logging
 import os
 from importlib import import_module
 
@@ -16,8 +17,12 @@ import climetlab
 from climetlab.core import Base
 from climetlab.core.metadata import annotate
 from climetlab.core.plugins import find_plugin, register
+from climetlab.core.settings import SETTINGS
+from climetlab.utils import download_and_cache
 from climetlab.utils.html import table
 
+LOG = logging.getLogger(__name__)
+
 
 class Dataset(Base):
     """Mother class to create a dataset object."""
@@ -96,22 +101,25 @@ class Dataset(Base):
     def __getitem__(self, n):
         return self.source[n]
 
-    def sel(self, *args, **kwargs):
-        return self.source.sel(*args, **kwargs)
+    def sel(self, **kwargs):
+        return self.source.sel(**kwargs)
 
-    def to_numpy(self, *args, **kwargs):
+    def to_numpy(self, **kwargs):
         import numpy as np
 
-        return np.array([s.to_numpy(*args, **kwargs) for s in self.source])
+        return np.array([s.to_numpy(**kwargs) for s in self.source])
 
-    def to_xarray(self, *args, **kwargs):
-        return self.source.to_xarray(*args, **kwargs)
+    def to_xarray(self, **kwargs):
+        return self.source.to_xarray(**kwargs)
 
-    def to_pandas(self, *args, **kwargs):
-        return self.source.to_pandas(*args, **kwargs)
+    def to_tfdataset(self, **kwargs):
+        return self.source.to_tfdataset(**kwargs)
 
-    def to_metview(self, *args, **kwargs):
-        return self.source.to_metview(*args, **kwargs)
+    def to_pandas(self, **kwargs):
+        return self.source.to_pandas(**kwargs)
+
+    def to_metview(self, **kwargs):
+        return self.source.to_metview(**kwargs)
 
     def _repr_html_(self):
         return table(self)
@@ -119,12 +127,6 @@ class Dataset(Base):
     def annotate(self, data, **kargs):
         return annotate(data, self, **kargs)
 
-    def read_csv_options(self, *args, **kwargs):
-        return {}
-
-    def read_zarr_options(self, *args, **kwargs):
-        return {}
-
 
 def _module_callback(plugin):
     return import_module(plugin, package=__name__).dataset
@@ -149,19 +151,38 @@ class YamlDefinedDataset(Dataset):
         return self
 
 
+def _dataset_from_dict(name, dataset, path=None):
+    attributes = dataset.get("metadata", {})
+    attributes.update(
+        dict(_path=path, _src=dataset["source"], _args=dataset.get("args", {}))
+    )
+    return type(camel(name), (YamlDefinedDataset,), attributes)
+
+
+def dataset_from_dict(path, *args, **kwargs):
+    return _dataset_from_dict(path)(*args, **kwargs).mutate()
+
+
+def _dataset_from_yaml(path):
+    name, _ = os.path.splitext(os.path.basename(path))
+    with open(path) as f:
+        dataset = yaml.load(f.read(), Loader=yaml.SafeLoader)["dataset"]
+        return _dataset_from_dict(name, dataset, path)
+
+
+def dataset_from_yaml(path, *args, **kwargs):
+    return _dataset_from_yaml(path)(*args, **kwargs).mutate()
+
+
 class DatasetLoader:
 
     kind = "dataset"
 
+    def settings(self, name):
+        return SETTINGS.get(name)
+
     def load_yaml(self, path):
-        name, _ = os.path.splitext(os.path.basename(path))
-        with open(path) as f:
-            dataset = yaml.load(f.read(), Loader=yaml.SafeLoader)["dataset"]
-            attributes = dataset.get("metadata", {})
-            attributes.update(
-                dict(_path=path, _src=dataset["source"], _args=dataset.get("args", {}))
-            )
-            return type(camel(name), (YamlDefinedDataset,), attributes)
+        return _dataset_from_yaml(path)
 
     def load_module(self, module):
         return import_module(module, package=__name__).dataset
@@ -172,6 +193,18 @@ class DatasetLoader:
             return entry
         return entry.dataset
 
+    def load_remote(self, name):
+        catalogs = self.settings("datasets-catalogs-urls")
+        for catalog in catalogs:
+            url = f"{catalog}/{name}.yaml"
+            path = download_and_cache(url, return_none_on_404=True)
+
+            if path:
+                LOG.debug("Found dataset at %s", url)
+                return self.load_yaml(path)
+
+        return None
+
 
 def register_dataset(module):
     register("dataset", module)
