@@ -7,17 +7,26 @@
 # nor does it submit to any jurisdiction.
 #
 
+import logging
 import os
-import warnings
 import weakref
 from importlib import import_module
 
 from climetlab.core import Base
 from climetlab.decorators import locked
 
+LOG = logging.getLogger(__name__)
+
 
 class Reader(Base):
+
+    appendable = False  # Set to True if the data can be appened to and existing file
+    binary = True
+
     def __init__(self, source, path):
+
+        LOG.debug("Reader for %s is %s", path, self.__class__.__name__)
+
         self._source = weakref.ref(source)
         self.path = path
 
@@ -25,10 +34,26 @@ class Reader(Base):
     def source(self):
         return self._source()
 
+    @property
+    def filter(self):
+        return self.source.filter
+
+    @property
+    def merger(self):
+        return self.source.merger
+
     def mutate(self):
         # Give a chance to `directory` or `zip` to change the reader
         return self
 
+    def mutate_source(self):
+        # The source may ask if it needs to mutate
+        return None
+
+    def ignore(self):
+        # Used by multi-source
+        return False
+
     def sel(self, *args, **kwargs):
         raise NotImplementedError()
 
@@ -39,34 +64,21 @@ class Reader(Base):
     def cache_file(self, *args, **kwargs):
         return self.source.cache_file(*args, **kwargs)
 
+    def save(self, path):
+        mode = "wb" if self.binary else "w"
+        with open(path, mode) as f:
+            self.write(f)
 
-class DefaultMerger:
-    def __init__(self, engine, backend_kwargs):
-        self.engine = engine
-        self.backend_kwargs = backend_kwargs
-
-    def merge(self, paths, **kwargs):
-        import xarray as xr
-
-        options = dict(backend_kwargs=self.backend_kwargs)
-        options.update(kwargs)
-        return xr.open_mfdataset(
-            paths,
-            engine=self.engine,
-            **options,
-        )
-
-
-class MultiReaders(Base):
-    backend_kwargs = {}
-
-    def __init__(self, readers):
-        self.readers = readers
-
-    def to_xarray(self, merger=None, **kwargs):
-        if merger is None:
-            merger = DefaultMerger(self.engine, self.backend_kwargs)
-        return merger.merge([r.path for r in self.readers], **kwargs)
+    def write(self, f):
+        if not self.appendable:
+            assert f.tell() == 0
+        mode = "rb" if self.binary else "r"
+        with open(self.path, mode) as g:
+            while True:
+                chunk = g.read(1024 * 1024)
+                if not chunk:
+                    break
+                f.write(chunk)
 
 
 _READERS = {}
@@ -81,28 +93,37 @@ def _readers():
             if path.endswith(".py") and path[0] not in ("_", "."):
                 name, _ = os.path.splitext(path)
                 try:
-                    _READERS[name] = import_module(f".{name}", package=__name__).reader
-                except Exception as e:
-                    warnings.warn(f"Error loading wrapper {name}: {e}")
+                    module = import_module(f".{name}", package=__name__)
+                    if hasattr(module, "reader"):
+                        _READERS[name] = module.reader
+                except Exception:
+                    LOG.exception("Error loading reader %s", name)
     return _READERS
 
 
 def reader(source, path):
 
+    assert isinstance(path, str), source
+
     if os.path.isdir(path):
         from .directory import DirectoryReader
 
         return DirectoryReader(source, path).mutate()
+    LOG.debug("Reader for %s", path)
 
     with open(path, "rb") as f:
         magic = f.read(8)
 
-    for name, r in _readers().items():
-        try:
-            reader = r(source, path, magic)
+    LOG.debug("Looking for a reader for %s (%s)", path, magic)
+
+    for deeper_check in (False, True):
+        # We do two passes, the second one
+        # allow the plugin to look deeper in the file
+        for name, r in _readers().items():
+            reader = r(source, path, magic, deeper_check)
             if reader is not None:
                 return reader.mutate()
-        except Exception as e:
-            warnings.warn(f"Error calling reader '{name}': {e}")
 
-    raise ValueError(f"Cannot find a reader for file '{path}' (magic {magic})")
+    from .unknown import Unknown
+
+    return Unknown(source, path, magic)
