@@ -30,6 +30,7 @@ class Source(Base):
     citation = "-"
 
     _dataset = None
+    _parent = None
 
     def __init__(self, **kwargs):
         self._kwargs = kwargs
@@ -41,13 +42,21 @@ class Source(Base):
         # Give a chance to `multi` to change source
         return self
 
-    def cache_file(self, create, args, extension=".cache"):
+    def flatten(self):
+        # Used by multi-source
+        return [self]
+
+    def ignore(self):
+        # Used by multi-source
+        return False
+
+    def cache_file(self, *args, **kwargs):
         owner = self.name
         if self.dataset:
             owner = self.dataset.name
         if owner is None:
             owner = self.__class__.__name__.lower()
-        return cache_file(owner, create, args, extension)
+        return cache_file(owner, *args, **kwargs)
 
     @property
     def dataset(self):
@@ -62,23 +71,29 @@ class Source(Base):
     def _set_dataset(self, dataset):
         self._dataset = dataset
 
+    @property
+    def parent(self):
+        if self._parent is None:
+            return None
+        return self._parent()
+
+    @parent.setter
+    def parent(self, parent):
+        self._set_parent(weakref.ref(parent))
+
+    def _set_parent(self, parent):
+        self._parent = parent
+
     def _repr_html_(self):
         return table(self)
 
-    def read_csv_options(self, *args, **kwargs):
-        if self.dataset is None:
-            return {}
-        return self.dataset.read_csv_options(*args, **kwargs)
-
-    def read_zarr_options(self, *args, **kwargs):
-        if self.dataset is None:
-            return {}
-        return self.dataset.read_zarr_options(*args, **kwargs)
-
     @classmethod
     def multi_merge(cls, sources):
         return None
 
+    def graph(self, depth=0):
+        print(" " * depth, self)
+
 
 class SourceLoader:
 
@@ -93,6 +108,9 @@ class SourceLoader:
             return entry
         return entry.source
 
+    def load_remote(self, name):
+        return None
+
 
 class SourceMaker:
     def __call__(self, name, *args, **kwargs):
@@ -117,19 +135,7 @@ def register_source(module):
     register("source", module)
 
 
-def load_source(name: str, *args, **kwargs) -> Source:
-    """Loads a data source.
-
-    Parameters
-    ----------
-    name : str
-        [description]
-
-    Returns
-    -------
-    Source
-        [description]
-    """
+def load_source(name: str, *args, context=None, **kwargs) -> Source:
     return source(name, *args, **kwargs).mutate()
 
 
