@@ -13,7 +13,8 @@ import ecmwfapi
 
 from climetlab.normalize import normalize_args
 
-from .base import APIKeyPrompt, FileSource
+from .file import FileSource
+from .prompt import APIKeyPrompt
 
 
 class MARSAPI(APIKeyPrompt):
@@ -94,8 +95,9 @@ class MARSRetriever(FileSource):
     def request(self, **kwargs):
         return kwargs
 
-    def read_csv_options(self):
-        return dict(
+    def to_pandas(self, **kwargs):
+
+        pandas_read_csv_kwargs = dict(
             sep="\t",
             comment="#",
             # parse_dates=["report_timestamp"],
@@ -104,5 +106,19 @@ class MARSRetriever(FileSource):
             compression="zip",
         )
 
+        pandas_read_csv_kwargs.update(kwargs.get("pandas_read_csv_kwargs", {}))
+
+        odc_read_odb_kwargs = dict(
+            # TODO
+        )
+
+        odc_read_odb_kwargs.update(kwargs.get("odc_read_odb_kwargs", {}))
+
+        return super().to_pandas(
+            pandas_read_csv_kwargs=pandas_read_csv_kwargs,
+            odc_read_odb_kwargs=odc_read_odb_kwargs,
+            **kwargs,
+        )
+
 
 source = MARSRetriever
