@@ -10,20 +10,41 @@
 import itertools
 from collections import defaultdict
 
+from climetlab.sources.empty import EmptySource
+
 from . import Source
 
 
 class MultiSource(Source):
-    def __init__(self, *sources, merger=None):
+    def __init__(self, *sources, filter=None, merger=None):
         if len(sources) == 1 and isinstance(sources[0], list):
             sources = sources[0]
+
+        # src = []
+        # for s in sources:
+        #     s = s.mutate()  # Just in case
+        #     if not s.ignore():
+        #         src += s.flatten()
+
+        self.sources = [s.mutate() for s in sources]
+        self.filter = filter
         self.merger = merger
-        self.sources = sources
-        self._lengths = [None] * len(sources)
+        self._lengths = [None] * len(self.sources)
+
+    def ignore(self):
+        return len(self.sources) == 0
+
+    def flatten(self):
+        return self.sources
 
     def mutate(self):
+
         if len(self.sources) == 1 and self.merger is None:
             return self.sources[0].mutate()
+
+        if len(self.sources) == 0:
+            return EmptySource()
+
         return self
 
     def _set_dataset(self, dataset):
@@ -59,11 +80,15 @@ class MultiSource(Source):
     def to_xarray(self, **kwargs):
         import xarray as xr
 
-        merged = self.sources[0].multi_merge(self.sources)
+        sources = self.sources
+        if len(sources) == 1:
+            return sources[0].to_xarray(**kwargs)
+
+        merged = sources[0].multi_merge(sources)
         if merged is not None:
             return merged.to_xarray(merger=self.merger, **kwargs)
 
-        arrays = [s.to_xarray() for s in self.sources]
+        arrays = [s.to_xarray() for s in sources]
 
         # Get values of scalar coordinates
         values = defaultdict(set)
@@ -83,5 +108,43 @@ class MultiSource(Source):
 
         return xr.merge(arrays)
 
+    def to_tfdataset(self, **kwargs):
+        sources = self.sources
+
+        merged = sources[0].multi_merge(sources)
+        if merged is not None:
+            return merged.to_tfdataset(merger=self.merger, **kwargs)
+
+        ds = sources[0].to_tfdataset()
+        for s in sources[1:]:
+            ds = ds.concatenate(s.to_tfdataset())
+
+        return ds
+
+    def to_pandas(self, **kwargs):
+        # sources = self.sources
+
+        # merged = sources[0].multi_merge(sources)
+        # if merged is not None:
+        #     return merged.to_tfdataset(merger=self.merger, **kwargs)
+
+        import pandas
+
+        return pandas.concat([s.to_pandas() for s in self.sources])
+
+    def __repr__(self) -> str:
+        string = ",".join(repr(s) for s in self.sources)
+        return f"{self.__class__.__name__}({string})"
+
+    def save(self, path):
+        with open(path, "wb") as f:
+            for s in self.sources:
+                s.write(f)
+
+    def graph(self, depth=0):
+        print(" " * depth, self.__class__.__name__)
+        for s in self.sources:
+            s.graph(depth + 3)
+
 
 source = MultiSource
